

/** CustOrdClient.java **/


import java.util.*;
import java.text.SimpleDateFormat;
import java.text.DecimalFormat;
import com.progress.open4gl.*;
import com.progress.open4gl.javaproxy.Connection;
import com.progress.open4gl.dynamicapi.*;
import org.eclipse.emf.ecore.*;
import org.eclipse.emf.ecore.sdo.*;
import org.eclipse.emf.ecore.sdo.impl.*;
import org.eclipse.emf.ecore.util.*;
import commonj.sdo.*;

import CustOrdersProxy.*;


public class CustOrdClient
{

  private static SimpleDateFormat m_sdfDate;


  public static void main(String[] args) throws Exception, ProDataException
  {
    CustOrders    appObj;
    Connection    connObj;
    String appServerURL = "AppServer://localhost/OCBroker";

    m_sdfDate = new SimpleDateFormat("yyyy-MM-dd");

    try  // To catch all exceptions
    {
      ProDataGraphHolder dgHolder = new ProDataGraphHolder();
      ProDataGraph dg = null;
      StringHolder errStrH = new StringHolder();
      String errMsg = null;

      connObj = new Connection(appServerURL, "", "", "");
      appObj = new CustOrders(connObj);
      System.out.println("Connected to AppServer OK");

      // Will get customer order rows for the first 5 customers
      appObj.getcustords(5, dgHolder, errStrH);
      errMsg = errStrH.getStringValue();
      if ((errMsg != null) && (errMsg.length() > 0))
      {
          System.out.println("ERROR: " + errMsg);
          appObj._release();
          return;
      }

      dg = dgHolder.getProDataGraphValue();

      outputRowCounts(dg);
      outputData(dg);

      appObj._release();

    } // try to catch all unexpected exceptions
    catch (ProDataException e)
    {
        System.out.println("Exception Message: " + e.getMessage());
    }
    catch (Exception e)
    {
        System.out.println("Exception Message: " + e.getMessage());
    }

  } // class CustOrdClient


  // For each table in the specified datagraph, outputs the number of rows
  public static void outputRowCounts(ProDataGraph mydg) throws Exception, ProDataException
  {
    ProDataGraphMetaData dgmd = mydg.getMetaData();
    String[] names = dgmd.getTableNames();

    for (int i = 0; i < mydg.getNumTables(); i++)
    {
          List doList = mydg.getProDataObjects(i);
          System.out.println();
          if (doList.size() > 0)
              System.out.println(names[i] + " Table has " + doList.size() + " rows" );
          else
              System.out.println(names[i] + " Table has no rows"  );
    }
    System.out.println();
  }

  // For each table in datagraph, print out schema and row data info to the console
  public static void outputData(ProDataGraph mydg) throws Exception, ProDataException
  {
    for (int i = 0; i < mydg.getNumTables(); i++)
    {
        outputTableData(mydg, i);
        System.out.println();
      }
  }

  public static void outputTableData(ProDataGraph dg, int tblIdx) throws Exception
  {
        ProDataGraphMetaData dgmd = dg.getMetaData();
        String[] names = dgmd.getTableNames();
        // Meta Data for this table
        ProDataObjectMetaData domd = dgmd.getTableMetaData(tblIdx);
        // List of rows for this table
        List rowList = dg.getProDataObjects(tblIdx);

        int numCols = domd.getFieldCount();
        int rowNum = 1;
        String value = null;

        // First print out the Schema
        System.out.println("\nSchema for Table " + names[tblIdx] + ": ");
        System.out.println("colName         colType    colExtent");
        System.out.println("------------------------------------");
        for (int i = 0; i < numCols; i++)
        {
            String colName = domd.getFieldName(i);
            int colType = domd.getProType(i);
            int colExtent = domd.getExtent(i);

            String typeName;
            switch (colType)
            {
              case Parameter.PRO_CHARACTER:
                typeName = "CHAR       ";
                break;
              case Parameter.PRO_INTEGER:
                typeName = "INT        ";
                break;
              case Parameter.PRO_LOGICAL:
                typeName = "LOGICAL    ";
                break;
              case Parameter.PRO_DECIMAL:
                typeName = "DECIMAL    ";
                break;
              case Parameter.PRO_DATE:
                typeName = "DATE       ";
                break;
              case Parameter.PRO_RAW:
                typeName = "RAW        ";
                break;
              case Parameter.PRO_DATETIME:
                typeName = "DATETIME   ";
                break;
              case Parameter.PRO_DATETIMETZ:
                typeName = "DATETIME-TZ";
                break;
              case Parameter.PRO_BLOB:
                typeName = "BLOB       ";
                break;
              case Parameter.PRO_CLOB:
                typeName = "CLOB       ";
                break;
              default:
                //shouldn't get here
                typeName = "UNKNOWN    ";
                break;
            }

            // Line up the strings
            int nameFill = 0;
            if (colName.length() < 16)
                nameFill = 16 - colName.length();

            System.out.print(colName);

            for (int j = 0; j < nameFill; ++j)
                System.out.print(" ");

            System.out.println(typeName + "  "+ colExtent);
        }

        System.out.println("\nData For Table " + names[tblIdx] + ": ");
        System.out.println("---------------------------------");


        // walk through the row list, extracting the data
        for (Iterator iter = rowList.iterator(); iter.hasNext();  )
        {
          ProDataObject dataObject = (ProDataObject)iter.next();

          for (int i = 0; i < numCols; i++)
          {
              String colName = domd.getFieldName(i);
              int colType = domd.getProType(i);

              Object obj = dataObject.get(i);

              if (obj == null)
              {
                  System.out.print("null    ");
              }
              else
              {
                switch(colType)
                {
                  case Parameter.PRO_DATE:
                    Date calDate = ((GregorianCalendar)obj).getTime();
                    value = m_sdfDate.format(calDate);
                  break;
                  case Parameter.PRO_DATETIME:
                  case Parameter.PRO_DATETIMETZ:
                    GregorianCalendar dtz = (GregorianCalendar) obj;
                    value = getDateTime(dtz);
                  break;
                  default:
                    value = obj.toString();
                  break;
                }
                System.out.print(value + "    ");
              }
          }
          System.out.println();
        }
        System.out.println();

  } // End outputTableData

  /**
     * Takes a GregorianCalendar object and generates a string that is
     * a proper xml dateTime using the date and time from the input.
     *
     * @param  dtz       - a GregorianCalendar to be turned into xml
     * @return fdateTime - a String whose data is the xml Schema dateTime data
     * @throws Exception
     **/
    private static String getDateTime(GregorianCalendar dtz)
    {
        DecimalFormat yearDigits  = new DecimalFormat("#0000;-#0000");
        DecimalFormat twoDigits   = new DecimalFormat("00");
        DecimalFormat threeDigits = new DecimalFormat("000");
        StringBuffer  fdateTime   = new StringBuffer();
        int iVal;

        iVal = dtz.get(Calendar.YEAR);
        fdateTime.append(yearDigits.format(iVal) + "-");
        iVal = dtz.get(Calendar.MONTH);
        fdateTime.append(twoDigits.format(iVal + 1) + "-");
        iVal = dtz.get(Calendar.DAY_OF_MONTH);
        fdateTime.append(twoDigits.format(iVal) + "T");
        iVal = dtz.get(Calendar.HOUR_OF_DAY);
        fdateTime.append(twoDigits.format(iVal) + ":");
        iVal = dtz.get(Calendar.MINUTE);
        fdateTime.append(twoDigits.format(iVal) + ":");
        iVal = dtz.get(Calendar.SECOND);
        fdateTime.append(twoDigits.format(iVal) + ".");
        iVal = dtz.get(Calendar.MILLISECOND);
        fdateTime.append(threeDigits.format(iVal));
        // account for daylight savings in the timezone
        int tzOff = dtz.get(Calendar.ZONE_OFFSET);
        int dstOff = dtz.get(Calendar.DST_OFFSET);
        iVal = (tzOff + dstOff) / 1000;
        if (iVal < 0)
        {
            fdateTime.append("-");
          iVal *= -1;
        }
        else fdateTime.append("+");
        fdateTime.append(twoDigits.format(iVal / 3600) + ":");
        fdateTime.append(twoDigits.format(((iVal / 60) % 60)) );
        return fdateTime.toString();
    }


} // main()



