

/** UpdateDSClient.java **/


import java.util.*;
import java.text.SimpleDateFormat;
import java.text.DecimalFormat;
import com.progress.open4gl.*;
import com.progress.open4gl.javaproxy.Connection;
import com.progress.open4gl.dynamicapi.*;
import org.eclipse.emf.ecore.*;
import org.eclipse.emf.ecore.sdo.*;
import org.eclipse.emf.ecore.sdo.impl.*;
import org.eclipse.emf.ecore.util.*;
import commonj.sdo.*;

import UpdateProxy.*;


public class UpdateDSClient
{

  private static SimpleDateFormat m_sdfDate = new SimpleDateFormat("yyyy-MM-dd");


  public static void main(String[] args) throws Exception, ProDataException
  {
    ProDataGraphHolder dgHolder = new ProDataGraphHolder();
    ProDataGraph dg = null, chgDG = null, origDG = null;
    UpdateAO appObj;
    Connection connObj;
    ProChangeSummary chgSumm;
    String appServerURL = "AppServer://localhost/OCBroker";
    int ret = 0;

    try  // To catch all exceptions
    {
      connObj = new Connection(appServerURL, "", "", "");
      appObj = new UpdateAO(connObj);
      System.out.println("Connected to AppServer OK");

      // Gets Order and Orderline rows with OrderNumbers in specified range
      int lowRange = 4;
      int highRange = 6;
      appObj.StaticDS(lowRange, highRange, dgHolder);

      // Retreive returned datagraph from holder
      origDG = dgHolder.getProDataGraphValue();

      System.out.println("\nOriginal DataSet");
      System.out.println("----------------");
      outputData(origDG, true);
      System.out.println();

      // Make some changes to the datagraph
      updateDS(origDG);

      // Now want to pass the changes back to the AppServer, but only want to
      // pass back the rows that have changed.
      chgSumm = origDG.getProChangeSummary();
      chgDG = chgSumm.getChanges();

      // Now pass changes back to AppServer
      dgHolder.setProDataGraphValue(chgDG);
      boolean ignoreDBChange = false;
      boolean mergeByField = true;
      appObj.UpdateDS(ignoreDBChange, mergeByField, dgHolder);

      // The returned ProDataGraph contains the changes sent, including any further updates to
      // these records or errors from the AppServer
      dg = dgHolder.getProDataGraphValue();

      System.out.println("Back from AppServer");
      System.out.println("---------------");
      outputData(dg, false);

      // Now reconcile returned data from AppServer
      ret = reconcileChanges(chgDG, dg);

      if (ret == 0)
      {
          dg.acceptChanges();
          System.out.println("Merged DataSet");
          System.out.println("---------------");
          outputData(origDG, false);
      }

      appObj._release();

    } // try to catch all unexpected exceptions
    catch (ProDataException e)
    {
        System.out.println("Exception Message: " + e.getMessage());
    }
    catch (Exception e)
    {
        System.out.println("Exception Message: " + e.getMessage());
    }

  } // class UpdateDSClient

  // Makes updates to specified datagraph
  private static void updateDS(ProDataGraph dg) throws Exception, ProDataException
  {
      ProChangeSummary chgSumm = dg.getProChangeSummary();
      ProDataGraphMetaData dgmd = dg.getMetaData();

      chgSumm.beginLogging();

      // DataGraph has 2 tables: Orders and OrderLines
      java.util.List orderList = dg.getProDataObjects(0);

      // For customer number 1, want to update credit card info
      for (int i = 0; i < orderList.size(); i++)
      {
          ProDataObject orderDataObj = (ProDataObject)orderList.get(i);
          int custNum = orderDataObj.getInt(1);  // custnum field

          if (custNum == 1)
          {
              orderDataObj.setString(14, "PREMIUM GOLD CARD");

              // Form Customer number 1, also want to change the quantity and status of all orders.
              // getChildRows() returns list of child ProDataObjects for specified relation name
              java.util.List orderLineList = orderDataObj.getChildRows("OrderLine");
              for (int j = 0; j < orderLineList.size(); j++)
              {
                  ProDataObject orderLineDataObj = (ProDataObject)orderLineList.get(j);
                  int qty = orderLineDataObj.getInt(4); //quantity field
                  orderLineDataObj.setInt(4, qty * 2);
                  orderLineDataObj.setString(7, "IN TRANSIT");
              }
          }
      }
      chgSumm.endLogging();
  }
  // Reconcile changes
  // origDG is the changes-only datagraph that was sent to the AppServer
  // dg is the changes-only datagraph returned from the AppServer including any further changes
  // or errors returned from AppServer
  // This method assumes there are no added or deleted rows
  private static int reconcileChanges(ProDataGraph origDG, ProDataGraph dg) throws Exception, ProDataException
  {
      ProDataGraphMetaData dgmd = dg.getMetaData();
      String[] names = dgmd.getTableNames();

      // If any errors returned from AppServer, throwing out all changes
      if (dg.hasError())
      {
          for (int i = 0; i < dg.getNumTables(); i++)
          {
              // ProDataObjectMetaData domd = dgmd.getTableMetaData(tblIdx);
              List rowList = dg.getProDataObjects(i);
              for (int j = 0; j < rowList.size(); j++)
              {
                  ProDataObject rowObj = (ProDataObject)rowList.get(j);

                  if (j == 0)
                      if (rowObj.hasTableError())
                      {
                          System.out.println("\nTable " + names[i] + " ERROR: " + rowObj.getTableErrorString());
                          System.out.println("\nThe following row(s) were not updated successfully:");
                      }

                  if (rowObj.hasRowError())
                      System.out.println("\nRow " + j + ": " + rowObj.getRowErrorString());
              }
          }
          return -1;
      }

      // SDO interface does not provide any merge facility. Doing manual merge to original
      // datagraph here. Need to do this merge in case rows were changed on AppServer.

      java.util.List orderList = dg.getProDataObjects(0);
      java.util.List origOrderList = origDG.getProDataObjects(0);

      for (int i = 0; i < orderList.size(); i++)
      {
          ProDataObject orderDataObj = (ProDataObject)orderList.get(i);
          ProDataObject origOrderDataObj = (ProDataObject)origOrderList.get(i);
          int custNum = orderDataObj.getInt(1);  // custnum field

          if (custNum == 1)
          {
              String str = orderDataObj.getString(14);  // credit card field
              String origStr = origOrderDataObj.getString(14);
              if (str.compareTo(origStr) != 0)
                  origOrderDataObj.setString(14, str);

              java.util.List oLineList = orderDataObj.getChildRows("OrderLine");
              java.util.List origOLineList = origOrderDataObj.getChildRows("OrderLine");
              for (int j = 0; j < oLineList.size(); j++)
              {
                  ProDataObject oLineDataObj = (ProDataObject)oLineList.get(j);
                  ProDataObject origOLineDataObj = (ProDataObject)origOLineList.get(j);
                  int qty = oLineDataObj.getInt(4); //quantity field
                  if (qty != origOLineDataObj.getInt(4))
                      origOLineDataObj.setInt(4, qty);
              }
          }
      }
      return 0;
  }

  // For each table in datagraph, print out schema and row data info to the console
  private static void outputData(ProDataGraph dg, boolean outputSchema) throws Exception, ProDataException
  {
    for (int i = 0; i < dg.getNumTables(); i++)
    {
        outputTableData(dg, i, outputSchema);
        System.out.println();
    }
  }

  private static void outputTableData(ProDataGraph dg, int tblIdx, boolean outputSchema) throws Exception
  {
        ProDataGraphMetaData dgmd = dg.getMetaData();
        String[] names = dgmd.getTableNames();
        // Meta Data for this table
        ProDataObjectMetaData domd = dgmd.getTableMetaData(tblIdx);
        // List of rows for this table
        List rowList = dg.getProDataObjects(tblIdx);

        int numCols = domd.getFieldCount();
        String value = null;

         if (outputSchema == true)
            outputSchema(dg, tblIdx);

        System.out.println("\nData For Table " + names[tblIdx] + ": ");
        System.out.println("---------------------------------");

        // walk through the row list, extracting the data
        for (Iterator iter = rowList.iterator(); iter.hasNext();  )
        {
          ProDataObject dataObject = (ProDataObject)iter.next();

          for (int i = 0; i < numCols; i++)
          {
              String colName = domd.getFieldName(i);
              int colType = domd.getProType(i);

              Object obj = dataObject.get(i);

              if (obj == null)
              {
                  System.out.print("null    ");
              }
              else
              {
                switch(colType)
                {
                  case Parameter.PRO_DATE:
                    Date calDate = ((GregorianCalendar)obj).getTime();
                    value = m_sdfDate.format(calDate);
                  break;
                  case Parameter.PRO_DATETIME:
                  case Parameter.PRO_DATETIMETZ:
                    GregorianCalendar dtz = (GregorianCalendar) obj;
                    value = getDateTime(dtz);
                  break;
                  default:
                    value = obj.toString();
                  break;
                }
                System.out.print(value + "    ");
              }
          }
          System.out.println();
        }
        System.out.println();

  } // End outputTableData

  public static void outputSchema(ProDataGraph dg, int tblIdx) throws Exception
  {
        ProDataGraphMetaData dgmd = dg.getMetaData();
        String[] names = dgmd.getTableNames();
        // Meta Data for this table
        ProDataObjectMetaData domd = dgmd.getTableMetaData(tblIdx);
        int numCols = domd.getFieldCount();

        // First print out the Schema
        System.out.println("\nSchema for Table " + names[tblIdx] + ": ");
        System.out.println("colName         colType    colExtent");
        System.out.println("------------------------------------");
        for (int i = 0; i < numCols; i++)
        {
            String colName = domd.getFieldName(i);
            int colType = domd.getProType(i);
            int colExtent = domd.getExtent(i);

            String typeName;
            switch (colType)
            {
              case Parameter.PRO_CHARACTER:
                typeName = "CHAR       ";
                break;
              case Parameter.PRO_INTEGER:
                typeName = "INT        ";
                break;
              case Parameter.PRO_LOGICAL:
                typeName = "LOGICAL    ";
                break;
              case Parameter.PRO_DECIMAL:
                typeName = "DECIMAL    ";
                break;
              case Parameter.PRO_DATE:
                typeName = "DATE       ";
                break;
              case Parameter.PRO_RAW:
                typeName = "RAW        ";
                break;
              case Parameter.PRO_DATETIME:
                typeName = "DATETIME   ";
                break;
              case Parameter.PRO_DATETIMETZ:
                typeName = "DATETIME-TZ";
                break;
              case Parameter.PRO_BLOB:
                typeName = "BLOB       ";
                break;
              case Parameter.PRO_CLOB:
                typeName = "CLOB       ";
                break;
              default:
                //shouldn't get here
                typeName = "UNKNOWN    ";
                break;
            }

            // Line up the strings
            int nameFill = 0;
            if (colName.length() < 16)
                nameFill = 16 - colName.length();

            System.out.print(colName);

            for (int j = 0; j < nameFill; ++j)
                System.out.print(" ");

            System.out.println(typeName + "  "+ colExtent);
        }

        System.out.println();

  } // End outputTableData


  /**
     * Takes a GregorianCalendar object and generates a string that is
     * a proper xml dateTime using the date and time from the input.
     *
     * @param  dtz       - a GregorianCalendar to be turned into xml
     * @return fdateTime - a String whose data is the xml Schema dateTime data
     * @throws Exception
     **/
    private static String getDateTime(GregorianCalendar dtz)
    {
        DecimalFormat yearDigits  = new DecimalFormat("#0000;-#0000");
        DecimalFormat twoDigits   = new DecimalFormat("00");
        DecimalFormat threeDigits = new DecimalFormat("000");
        StringBuffer  fdateTime   = new StringBuffer();
        int iVal;

        iVal = dtz.get(Calendar.YEAR);
        fdateTime.append(yearDigits.format(iVal) + "-");
        iVal = dtz.get(Calendar.MONTH);
        fdateTime.append(twoDigits.format(iVal + 1) + "-");
        iVal = dtz.get(Calendar.DAY_OF_MONTH);
        fdateTime.append(twoDigits.format(iVal) + "T");
        iVal = dtz.get(Calendar.HOUR_OF_DAY);
        fdateTime.append(twoDigits.format(iVal) + ":");
        iVal = dtz.get(Calendar.MINUTE);
        fdateTime.append(twoDigits.format(iVal) + ":");
        iVal = dtz.get(Calendar.SECOND);
        fdateTime.append(twoDigits.format(iVal) + ".");
        iVal = dtz.get(Calendar.MILLISECOND);
        fdateTime.append(threeDigits.format(iVal));
        // account for daylight savings in the timezone
        int tzOff = dtz.get(Calendar.ZONE_OFFSET);
        int dstOff = dtz.get(Calendar.DST_OFFSET);
        iVal = (tzOff + dstOff) / 1000;
        if (iVal < 0)
        {
            fdateTime.append("-");
          iVal *= -1;
        }
        else fdateTime.append("+");
        fdateTime.append(twoDigits.format(iVal / 3600) + ":");
        fdateTime.append(twoDigits.format(((iVal / 60) % 60)) );
        return fdateTime.toString();
    }


} // main()



